#!/usr/bin/python
# Rotor Control Python Utilities
# Copyright (C) 2006, 2007, 2008, 2009 Leigh L. Klotz, Jr <Leigh@WA5ZNU.org>
# Licensed under Academic Free License 3.0 (See LICENSE)

# TODO:
# Read bearing
# Move XML parsing to library
# Handle "a/b" rotations and talk to steppir

import string, serial, time, sys, math, os
from lxml import etree

CONFIGFILE="~/python/k3-python-rigcontrol/config/rotor.xml"

class Rotor:
  config = None

  def __init__(self):
    self.configig = etree.parse(os.path.expanduser(CONFIGFILE)).getroot()
    assert self.configig.tag == "rotor"
    self.device=self.configig.findtext("port/device")
    self.baudrate=int(self.configig.findtext("port/baud"))
    if (self.configig.findtext("offset")):
       self.offset=int(self.configig.findtext("offset"))
    else:
       self.offset=0
    self.ser = serial.Serial(self.device,baudrate=self.baudrate,rtscts=0) #timeout=2
    self.ser.flushOutput()

  def close(self):
   self.ser.close()

  #
  # Set the rotor bearing and rotate
  # API1xxx;AM1;
  def rotate(self,deg):
    cmd=self.lookupCommand("setbearing")
    (argp, fmtstring) = self.symbolToFormatString(cmd)
    deg = deg + self.offset
    if (deg >= 360):
     deg = deg - 360
    if (deg < 0):
     deg = deg + 360
    self.writeCommand(argp, fmtstring, deg)
    cmd=self.lookupCommand("rotate")
    (argp, fmtstring) = self.symbolToFormatString(cmd)
    self.writeCommand(argp, fmtstring)

  def rotateq(self):
    cmd=self.lookupCommand("bearing")
    (argp, fmtstring) = self.symbolToFormatString(cmd)
    self.writeCommand(argp, fmtstring)
    resp = self.lookupReply("bearing")  # crock
    result = int(self.readCommand(resp))
    result = result-self.offset
    if (result < 0):
     result = result + 360
    if (result >= 360):
     result = result - 360
    return result

  def writeCommand(self, argp, fmtstring, arg=""):
    if (argp):
     txt=fmtstring % arg
     #print txt
     self.ser.write(txt)
    else:
     txt=fmtstring
     #print txt
     self.ser.write(txt)

  def readCommand(self, resp):
   (argp, fmtstring) = self.symbolToFormatString(resp)
   dataformatsize = int(resp.xpath("size")[0].text)
   data = self.ser.read(dataformatsize)
   return data[1:]            # crock

  def lookupCommand(self, xp):
    return self.config.xpath("command[symbol='"+xp+"']")[0]

  #crock
  def lookupReply(self, config, xp):
    return self.self.configig.xpath("reply[symbol='"+xp+"']")[0]

  def symbolToFormatString(self, txt):
    fmtstring = ""
    argp=False
    for node in txt.xpath("*"):
      if (node.tag=='symbol'):
       pass
      elif (node.tag=='size'):
       pass
      elif (node.tag=='string'):
       fmtstring = fmtstring + node.text
      elif (node.tag=='byte'):
        fmtstring = fmtstring + chr(int(node.text, 16))
      elif (node.tag=='data'):
        argp=True
        dataformatsize = node.xpath("size")[0].text
        dataformat = node.xpath("dtype")[0].text
        if (dataformat == "decimal"):
          fmtstring = fmtstring +"%0"+dataformatsize+"d"
        else:
         raise "unknown data dtype " + dataformat
      elif (node.tag=='info'):
        pass
      else:
        raise "unknown command " + node.tag
    return (argp,fmtstring)


